package com.poststats.golf.rs.api;

import java.math.BigInteger;
import java.util.List;

import com.poststats.golf.rs.api.model.EventRound;
import com.poststats.golf.rs.api.model.EventRoundPairing;
import com.poststats.golf.rs.api.model.EventRoundPairingOrder;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Positive;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.StreamingOutput;

/**
 * @author brian.long@poststats.com
 */
@Path("/golf/event/{eventId}")
@Tag(name = "Event Round API")
public interface EventRoundApi {

	@GET
	@Path("/rounds/next")
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Retrieves limited meta-data about the next round in the specified event.",
			description = "Retreives name, location, dates, and other direct meta-data about the upcoming round in the specified event."
	)
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(
					responseCode = "404",
					description = "An event with the specified ID or upcoming event rounds could not be found"
			)
	})
	List<EventRound> getNext(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@GET
	@Path("/round/{eroundId:[0-9]+}")
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Retrieves limited meta-data about the specified event round.",
			description = "Retreives name, location, dates, and other direct meta-data about the specified event round."
	)
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(
					responseCode = "404",
					description = "An event or event round with the specified ID could not be found"
			)
	})
	EventRound getOne(
			@Parameter(description = "A unique identifier for an event")
			@NotNull @Positive @PathParam("eventId") long eventId,
			@Parameter(description = "A unique identifier for an event round")
			@NotNull @Positive @PathParam("eroundId") long eroundId);

	@GET
	@Path("/rounds")
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Retrieves limited meta-data about all the rounds in the specified event.",
			description = "Retreives name, location, dates, and other direct meta-data about all the rounds in the specified event."
	)
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(
					responseCode = "404",
					description = "An event with the specified ID or any event rounds could not be found"
			)
	})
	List<EventRound> getAll(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@GET
	@Path("/round/{eroundId:[0-9]+}/pairings")
	@Produces(Constants.V1_JSON)
	@Operation(summary = "Retrieves limited meta-data about all pairings for a single event round.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(
					responseCode = "404",
					description = "An event or its round with the specified ID could not be found"
			)
	})
	List<EventRoundPairing> getPairings(
			@Parameter(description = "A unique identifier for an event")
			@NotNull @Positive @PathParam("eventId") long eventId,
			@Parameter(description = "A unique identifier for an event round")
			@NotNull @Positive @PathParam("eroundId") long eroundId);

	@GET
	@Path("/round/{eroundId:[0-9]+}/pairings/csv")
	@Produces("text/csv")
	@Operation(summary = "Retrieves limited meta-data about all pairings for a single event round in the CSV format.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(
					responseCode = "404",
					description = "An event or its round with the specified ID could not be found"
			)
	})
	StreamingOutput getPairingsAsCsv(
			@Parameter(description = "A unique identifier for an event")
			@NotNull @Positive @PathParam("eventId") long eventId,
			@Parameter(description = "A unique identifier for an event round")
			@NotNull @Positive @PathParam("eroundId") long eroundId,
			@QueryParam("orderBy") List<EventRoundPairingOrder> orderBys,
			@QueryParam("lastNameFirst") boolean lastNameFirst);

	@GET
	@Path("/round/{eroundId:[0-9]+}/pairing/{pairingID:[0-9]+}")
	@Produces(Constants.V1_JSON)
	@Operation(summary = "Retrieves limited meta-data about an event round pairing.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(
					responseCode = "404",
					description = "An event with the specified ID or upcoming event rounds could not be found"
			)
	})
	EventRoundPairing getPairing(
			@Parameter(description = "A unique identifier for an event")
			@NotNull @Positive @PathParam("eventId") long eventId,
			@Parameter(description = "A unique identifier for an event round")
			@NotNull @Positive @PathParam("eroundId") long eroundId, @PathParam("pairingID") BigInteger pairingId);

}
